use std::sync::Arc;
use async_trait::async_trait;
use matrix_sdk::Client;
use postbus::{Handler, SmtpService, SmtpState, command::Mailbox};
use crate::config::Mapping;

mod matrix;
mod config;


#[tokio::main]
async fn main() {
    let args: Vec<String> = std::env::args().collect();
    if args.len() != 2 {
        panic!("The path to the configuration file must be supplied as (only) command line argument!");
    }
    let mut config = crate::config::load_config(&args[1]);
    let immutable_config = config.clone();
    let clients = matrix::get_clients(&mut config).await;
    crate::config::write_config(&config, &immutable_config, &args[1]);

    let service = SmtpService::create(format!("{}:{}", config.bind_address.unwrap(), config.bind_port.unwrap()).parse().unwrap(), "matrixmailer".into(), 
    Arc::new(ToMatrixConverter {
            receiver_whitelist: config.receiver_whitelist,
            matrix_clients: clients,
            mappings: config.mappings,
    }));
    service.listen().await;
}

struct ToMatrixConverter {
    // If None, any incoming mail will be accepted. 
    // If Some(x), any mail with a mail address in x in its recipients will be accepted.
    // "Not accepted" means the sender will get a "RCPT failed" reply
    receiver_whitelist: Option<Vec<String>>,
    matrix_clients: Vec<Client>,
    mappings: Vec<Mapping>,
}

#[async_trait]
impl Handler for ToMatrixConverter {
    // Checks whether receipient is local (and the mail should thus be handled by this service)
    async fn recipient_local(&self, recipient: &Mailbox) -> bool {
        match &self.receiver_whitelist {
            None => true,
            Some(list) => {
                let addr = format!("{}@{}", recipient.local, recipient.domain.0);
                for entry in list {
                    if entry.eq(&addr) {
                        return true;
                    }
                }
                false
            }
        }
    }
    async fn save(&self, state: &SmtpState) -> bool {
        for recipient in &state.recipients {
            matrix::send(&state.from, &recipient, &state.data, &self.matrix_clients, &self.mappings).await;
        }
        true
    }
}